
#ifndef SYSTEM_USER_DEFINES_H
#define SYSTEM_USER_DEFINES_H

#include <xc.h>
#include <stdint.h>



#define PWM_CLOCK   4000000000UL   // PWM Generator Base Clock Frequency in [Hz]

#define SWITCHFREQ  500000UL  // define your switching frequency ***DAVID***
#define PWM_PERIOD  8000 //((volatile uint16_t) (PWM_CLOCK/SWITCHFREQ))
//#define PWM_RES                     (1.0/PWM_CLOCK)         // PWM Resolution

#define PWM_DEAD_TIME_RE            28  // Falling edge dead time x[250ps]
#define PWM_DEAD_TIME_FE            28  // Falling edge dead time

#define MAXDUTYPERCENT  84.5 //units are % to establish the maximum duty-cycle  ***DAVID***
#define MINDUTYPERCENT  0.0 // %
#define DUTYMAX         ((unsigned const int) (PWM_PERIOD * MAXDUTYPERCENT / 100.0)) //
#define DUTYMIN         ((unsigned const int) (PWM_PERIOD * MINDUTYPERCENT / 100.0))
#define DUTY_FIXED      ((volatile uint16_t) (PWM_PERIOD * 80.0 / 100.0))
#define DUTY_FIXED2      ((volatile uint16_t) (PWM_PERIOD * 45.0 / 100.0))
//#define VCFLYDELTA      2 // Cfly voltage max allowed imbalance voltage [V]
#define VOUTACTUALVOLTS 12//20.05 //***DAVID***
#define VINACTUALVOLTS 64
//#define INDUCTORCURRENTRIPPLE ((const int) (3300.0/10.0/4096*32768.0*10.0))
//#define INDUCTORCURRENTRIPPLE 5.5 //A

#define ADC_TRIG_OFFSET    1380//2800//2672//1440
#define ADC_TRIG_OFFSET2   1380//2800//1904// 1200  // need to update this
//#define ADCSAMPLEDELAY PWM_PERIOD-600
//#define SADCSAMPLEDELAY PWM_PERIOD-2200
#define ILIMITACTUALAMPS 17 //***DAVID*** output current
#define ISENSEREF 0x800
#define ISENSEOFFSET1 2066
#define ISENSEOFFSET2 2036
#define VOUTKEEPOFFATSTARTUP 14.0  //***DAVID*** pre-bias
#define VINMINFOR5V 6.5
#define RAMPRATE 15 //***DAVID***  // 30 for 28V, 45 for 56V
#define MAXOFFSET 100 //***YZ*** max duty cycle imbalance units [250ps]
//#define MAXOFFSETNEG 51
//#define MAXOFFSETINC 20 // when in sub stable, max additional ducy cycle offset
#define UVLO_ON 14.0 //***DAVID***
#define UVLO_OFF 12.0 //***DAVID***
#define OVLO_ON 80.0 //***DAVID***
#define OVLO_OFF 85.0 //***DAVID*** //cannot exceed 69 due to max ADC value and divider
#define TEMPERATURESD 1550 //***DAVID***
#define TEMPERATUREON 1500 //***DAVID***

#define VREF 3.3
#define ADCSCALE (1<<12)  //12 bit ADC value
//#define MAXTEMPERATURE 850 //850 = 85degC
//#define TEMPERATUREOFFSET 620 //remove 0.5V offset
//#define TEMPERATURESCALE ((const int) (3300.0/10.0/4096*32768.0*10.0)) //scale is 0.1degC = 1
//#define CURRENTSCALE ((const int) (0.15/3.3*4096))

#define VOUTADC(X) ((const int) ((X * 2.7/(31.02+2.7)/VREF*ADCSCALE +0.5) )) //do not converter to Q15
#define VINADC(X) ((const int) ((X * 9.1/(180.0 + 9.1)/VREF * ADCSCALE + 0.5) )) //do not convert to Q15
#define IOUTADC(X) ((const int) (X * 0.1 * 0.5)/VREF * ADCSCALE ) //do not convert to Q15
#define VCFLYADC(X) ((const int) (X * 2.0/(39.0 + 2.0)/VREF * ADCSCALE + 0.5)) // do not convert to Q15



#define VFLY_P_IBIT     2   // 1/4 is Kp
#define VFLY_I_IBIT     8  // 1/512 is Ki
#define VFLY_I_LIMIT    20480

#define VOUT_I_LIMIT    32000

#define M_ASR16(X,Y) ((X >> Y) | (( (int)X < 0)?(0xffff << (16-Y)):0))  // shift right. use this in capacitor voltage PI loop to save time




#define PWMDISABLED { PG1IOCONH &= 0xFFF3; PG4IOCONH &= 0xFFF3; }
#define PWMENABLE   { PG1IOCONH |= 0x000C; PG4IOCONH |= 0x000C; }

#define PWMHOLD     { PG1IOCONL |= 0x3000; PG4IOCONL |= 0x3000; } // Set override bits 
#define PWMRELEASE  { PG1IOCONL &= 0xCFFF; PG4IOCONL &= 0xCFFF; } // clear override bits, PWML=Low


#define ENABLEINPUT PORTBbits.RB5

//// These macros are used for UART Transmission of Data
//#define BAUDRATE 		9600
//#define BRGVAL 		((FCY/BAUDRATE)/16)-1  //instruction cycle frequency = 70 MHz = fcy
//#define DELAY_105uS     asm volatile ("REPEAT, #4201"); Nop(); // 105uS delay
//#define DELAY_1uS     asm volatile ("REPEAT, #34"); Nop(); // 105uS delay was 69
//#define DELAY_5uS     asm volatile ("REPEAT, #349"); Nop(); // 105uS delay

/******************************************************************************/
/* User Function Prototypes                                                   */
/******************************************************************************/

//void InitApp(void);         /* I/O and Peripheral Initialization */

extern volatile uint16_t ConfigureOscillator(void);
extern volatile uint16_t ConfigureSystemTimer(void);
extern volatile uint16_t ConfigureGPIOs(void);
extern volatile uint16_t ConfigureUART(void);
extern volatile uint16_t ConfigureADC(void);
extern volatile uint16_t ConfigurePWM(void);
extern volatile uint16_t LaunchPWM(void);
extern volatile uint16_t ConfigureOPA (void);
extern volatile uint16_t ConfigureControlLoops(void);
extern volatile uint16_t ConfigureComparators(void);
extern volatile uint16_t PowerControlStateMachine(void);

extern void SendData(void);

//void FilterData(void);

// state machine for the whole converter
enum STATES {
    off,
    outputdischarged,
    waiting5v,
    chargeCfly,
    softstart_v,
    softstart,
    running,
    error
};

// state machine for determining the direction (positive or negative) for capacitor voltage balancing output signal (deltaDQ2)
enum CTRSTATES {
    v_only,
    vNi
};



#endif // end of SYSTEM_USER_DEFINES_H
